---------------------------------------------------------------------------------
-- Alumne: Miguel Alvarez Lopez (bluesea)
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO D'UN DATA WAREHOUSE PER UNA CADENA DE BOTIGUES DE ROBA
---------------------------------------------------------------------------------

/*****************************************************************
-- Nom:                 ALTAPRODUCTES
--
-- Descripci:          Dna d'alta un producte al catleg de la cadena de de botigues de roba
--
-- Tasca: CU06-01 
--

-- Parmetres entrada:
						pID_PROD PRODUCTES.ID_PROD%TYPE,
						pDESCRIPCIO PRODUCTES.DESCRIPCIO%TYPE,
						pDATA_CAT PRODUCTES.DATA_CAT%TYPE,
--
-- Error controlats:   - producte duplicat
--
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ALTAPRODUCTES" ( 
						pID_PROD PRODUCTES.ID_PROD%TYPE,
						pDESCRIPCIO PRODUCTES.DESCRIPCIO%TYPE,
						pDATA_CAT PRODUCTES.DATA_CAT%TYPE,
   						RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  PARAMS VARCHAR2(1024);
  
BEGIN

  BEGIN
    RESPOSTA:='OK';
      PARAMS:=pID_PROD||';'||pDESCRIPCIO||';'||pDATA_CAT;

      INSERT INTO PRODUCTES (ID_PROD,DESCRIPCIO,DATA_CAT) 
      VALUES (pID_PROD,pDESCRIPCIO,pDATA_CAT);
    
      ALTALOG ('ALTAPRODUCTES',PARAMS,RESPOSTA);
      	
 EXCEPTION  
        WHEN DUP_VAL_ON_INDEX THEN
          RESPOSTA := 'el producte ja existeix. ';
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);
            ALTALOG ('ALTAPRODUCTES',PARAMS,RESPOSTA);
		  
        WHEN OTHERS THEN
          RESPOSTA := 'Error en alta productes: (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);  
            ALTALOG ('ALTAPRODUCTES',PARAMS,RESPOSTA);
  END;

COMMIT;
END;
/

/*****************************************************************
-- Nom:                 MODIFICARPRODUCTES
--
-- Descripci:          Modifica les dades dun producte
--
-- Tasca: CU06-02
--
-- Parmetres entrada:
						pID_PROD PRODUCTES.ID_PROD%TYPE,
						pDESCRIPCIO PRODUCTES.DESCRIPCIO%TYPE,
						pDATA_CAT PRODUCTES.DATA_CAT%TYPE,
						pBENEFICI_NET PREU.BENEFICI_NET%TYPE

--
-- Error controlats:   - producte no trobat
--
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARPRODUCTES" ( 
						pID_PROD PRODUCTES.ID_PROD%TYPE,
						pDESCRIPCIO PRODUCTES.DESCRIPCIO%TYPE,
						pDATA_CAT PRODUCTES.DATA_CAT%TYPE,
	   	                                RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);

BEGIN
  
   BEGIN
    RESPOSTA:='OK';
	PARAMS:=pID_PROD||';'||pDESCRIPCIO||';'||pDATA_CAT;

	SELECT COUNT(*) INTO COUNT_TMP FROM PRODUCTES WHERE ID_PROD = pID_PROD;

	IF (COUNT_TMP = 0) THEN
        	RAISE NO_DATA_FOUND;
    	END IF;
    
    	UPDATE PRODUCTES
    	SET DESCRIPCIO = pDESCRIPCIO, 
            DATA_CAT = pDATA_CAT
        WHERE ID_PROD = pID_PROD;		

	ALTALOG ('MODIFICARPRODUCTES',PARAMS,RESPOSTA);

   EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap producte amb identificador proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   ALTALOG ('MODIFICARPRODUCTES',PARAMS,RESPOSTA);
		   
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci del producte : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
		  ALTALOG ('MODIFICARPRODUCTES',PARAMS,RESPOSTA);
  END;

COMMIT;
END;
/

/*****************************************************************
-- Nom:                 ESBORRARPRODUCTES
--
-- Descripci:          Esborra el producte, del catleg, de la base de dades de la cadena de botigues de roba
--
-- Tasca: CU06-03
--
-- Parmetres entrada:
						pID_PROD PRODUCTES.ID_PROD%TYPE
--
-- Error controlats:   - producte no trobat
--
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARPRODUCTES" ( 
						pID_PROD PRODUCTES.ID_PROD%TYPE,
					   	RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
  
BEGIN

  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pID_PROD;
			
	SELECT COUNT(*) INTO COUNT_TMP FROM PRODUCTES WHERE ID_PROD = pID_PROD;

    	IF (COUNT_TMP = 0) THEN
        	RAISE NO_DATA_FOUND;
    	END IF;
    
	DELETE FROM PRODUCTES WHERE ID_PROD = pID_PROD;

	ALTALOG ('ESBORRARPRODUCTES',PARAMS,RESPOSTA);

  EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap producte amb identificador proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   ALTALOG ('ESBORRARPRODUCTES',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant producte : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA); 
		  ALTALOG ('ESBORRARPRODUCTES',PARAMS,RESPOSTA);		  
  END;

COMMIT;
END;
/

