---------------------------------------------------------------------------------
-- Alumne: Miguel Alvarez Lopez (bluesea)
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO D'UN DATA WAREHOUSE PER UNA CADENA DE BOTIGUES DE ROBA
---------------------------------------------------------------------------------


/*****************************************************************
-- Nom:                 ALTADWHVENDA
-
-- Descripci:          Dna d'alta una dwh venda feta a la cadena de de botigues de roba
--
-- Tasca: CU15-01 i CU16-01
--
-- Parmetres entrada:
						pID_BOTIGA DWH_VENDA.ID_BOTIGA%TYPE,
						pID_PROD DWH_VENDA.ID_PROD%TYPE,
						pID_DATA DWH_VENDA.ID_DATA%TYPE,
						pID_HORA DWH_VENDA.ID_HORA%TYPE,
						pQUANTITAT DWH_VENDA.QUANTITAT%TYPE,
						pPREU_BRUT DWH_VENDA.PREU_BRUT%TYPE,
						pBENEFICI_NET DWH_VENDA.BENEFICI_NET%TYPE,
--
-- Error controlats:   - hora format HH24
--
****************************************************************/
CREATE OR REPLACE PROCEDURE "ALTADWHVENDA" ( 
                                        pID_BOTIGA DWH_VENDA.ID_BOTIGA%TYPE,
                                        pID_PROD DWH_VENDA.ID_PROD%TYPE,
                                        pID_DATA DWH_VENDA.ID_DATA%TYPE,
                                        pID_HORA DWH_VENDA.ID_HORA%TYPE,
                                        pQUANTITAT DWH_VENDA.QUANTITAT%TYPE,
                                        pPREU_BRUT DWH_VENDA.PREU_BRUT%TYPE,
                                        pBENEFICI_NET DWH_VENDA.BENEFICI_NET%TYPE,
                                        RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);
      
BEGIN
  
  DECLARE
     EXCEP2_ERROR EXCEPTION;
          
  BEGIN
         
    PARAMS:=pID_BOTIGA||';'||pID_PROD||';'||pID_DATA||';'||pID_HORA||';'||pQUANTITAT||';'||pPREU_BRUT||';'||pBENEFICI_NET;
    
    -- Comprovem que la hora sigui correcte
    IF pID_HORA NOT BETWEEN 0 AND 23 THEN
       RAISE EXCEP2_ERROR;
    END IF;
    
    RESPOSTA:='OK';
 
  -- El primer que fem, abans d'inserir el registre, es comprovar si existeix
  -- ja un registre amb la mateixa PK.

    SELECT COUNT(*) INTO COUNT_TMP FROM DWH_VENDA WHERE ID_BOTIGA = pID_BOTIGA
                                             		AND ID_PROD = pID_PROD
                                               		AND ID_DATA = pID_DATA
                                               		AND ID_HORA = pID_HORA;

  -- Si NO existeix el registre a DWH_VENDA, l'inserim a la taula.

  IF (COUNT_TMP = 0) THEN	

		INSERT INTO DWH_VENDA (ID_BOTIGA,ID_PROD,ID_DATA,ID_HORA,QUANTITAT,PREU_BRUT,BENEFICI_NET) 
		VALUES (pID_BOTIGA,pID_PROD,pID_DATA,pID_HORA,pQUANTITAT,pPREU_BRUT,pBENEFICI_NET);
    
		ALTALOG ('ALTADWHVENDA_INS',PARAMS,RESPOSTA);

  -- Si SI existeix el registre a DWH_VENDA, nomes sumem la quantitat al 
  -- registre ja existent.
	
  ELSE
	UPDATE DWH_VENDA
	SET QUANTITAT = pQUANTITAT + QUANTITAT
		WHERE ID_BOTIGA = pID_BOTIGA
            		AND ID_PROD = pID_PROD
		        AND ID_DATA = pID_DATA
            		AND ID_HORA = pID_HORA;
		
	ALTALOG ('ALTADWHVENDA_SUMA',PARAMS,RESPOSTA);

	END IF;

 EXCEPTION 
    WHEN EXCEP2_ERROR THEN 
       RESPOSTA:='Hora incorrecte, format HH24';
       DBMS_OUTPUT.PUT_LINE(RESPOSTA);  
           ALTALOG ('ALTADWHVENDA',PARAMS,RESPOSTA);
  
    WHEN OTHERS THEN
       RESPOSTA := 'Error en alta-modificaci dwh_venda: (' || SQLCODE || ') ' || SQLERRM;
       DBMS_OUTPUT.PUT_LINE(RESPOSTA);  
           ALTALOG ('ALTADWHVENDA',PARAMS,RESPOSTA);
	
  END;

  -- ALTAVENDA_EST: alta a la taula DWH_EST en cas de que no existeixi cap 
  -- registre amb el camp ANYO = ANNY. Si existeix, o una vegada feta la 
  -- inserci, cridem a la procedure MOD_EST_I per fer els calculs
  -- estadstics i omplir la resta de camps de la taula DWH_EST.
	
  DECLARE 
    ANNY NUMBER(4);
  
  BEGIN
    -- Veure si existeix el registre a la taula DWH_EST amb l'any de la DWH_VENDA.

    ANNY := TO_CHAR(pID_DATA,'YYYY');  

    SELECT COUNT(*) INTO COUNT_TMP
	  FROM DWH_EST
	  WHERE ANYO = ANNY;

    RESPOSTA:='OK';  
    IF (COUNT_TMP = 0) THEN

        -- Si NO existeix, inserir el registre a DWH_EST amb l'any de la DWH_VENDA
        -- y als altres camps el valor NULL. Desprs cridem a la procedure 
        -- MOD_EST_I que far els clculs estadstics i omplir la resta de camps de la taula DWH_EST.

        INSERT INTO DWH_EST
        VALUES(ANNY,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
					
        MOD_EST_I(ANNY,RESPOSTA);	

    ELSE
	
	-- Si SI existeix,cridem a la procedure MOD_EST_I que far els clculs estadstics.
        MOD_EST_I(ANNY,RESPOSTA);

    END IF;

    EXCEPTION  
	WHEN OTHERS THEN
	   RESPOSTA := 'Error en : (' || SQLCODE || ') ' || SQLERRM;
	   DBMS_OUTPUT.PUT_LINE(RESPOSTA);  
 		    ALTALOG ('ALTAVENDA_EST',PARAMS,RESPOSTA);

    END; 

COMMIT;
END;
/

/*****************************************************************
-- Nom:                 MODIFICARDWHVENDA
--
-- Descripci:          Modifica les dades duna venda
--
-- Tasca: CU15-02 i CU16-02
--
-- Parmetres entrada:
						pID_BOTIGA VENDA.ID_BOTIGA%TYPE,
						pID_PROD VENDA.ID_PROD%TYPE,
						pID_DATA DWH_VENDA.ID_DATA%TYPE,
						pID_HORA DWH_VENDA.ID_HORA%TYPE,
						pQUANTITAT DWH_VENDA.QUANTITAT%TYPE,
						pPREU_BRUT DWH_VENDA.PREU_BRUT%TYPE,
						pBENEFICI_NET DWH_VENDA.BENEFICI_NET%TYPE,

--
-- Error controlats:   - venda no trobada
--
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MODIFICARDWHVENDA" ( 
						pID_BOTIGA DWH_VENDA.ID_BOTIGA%TYPE,
						pID_PROD DWH_VENDA.ID_PROD%TYPE,
						pID_DATA DWH_VENDA.ID_DATA%TYPE,
						pID_HORA DWH_VENDA.ID_HORA%TYPE,
						pQUANTITAT DWH_VENDA.QUANTITAT%TYPE,
						pPREU_BRUT DWH_VENDA.PREU_BRUT%TYPE,
						pBENEFICI_NET DWH_VENDA.BENEFICI_NET%TYPE,
					        RESPOSTA OUT NOCOPY VARCHAR2) 					   
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);

BEGIN
  
  BEGIN
    RESPOSTA:='OK';
	PARAMS:=pID_BOTIGA||';'||pID_PROD||';'||pID_DATA||';'||pID_HORA||';'||pQUANTITAT||';'||pPREU_BRUT||';'||pBENEFICI_NET;

  -- El primer que fem es comprovar si existeix ja un registre amb la mateixa PK.

    SELECT COUNT(*) INTO COUNT_TMP FROM DWH_VENDA WHERE ID_BOTIGA = pID_BOTIGA
                                             		AND ID_PROD = pID_PROD
                                               		AND ID_DATA = pID_DATA
                                               		AND ID_HORA = pID_HORA;
    IF (COUNT_TMP = 0) THEN
       	RAISE NO_DATA_FOUND;
    END IF;
    
    UPDATE DWH_VENDA
    SET QUANTITAT = pQUANTITAT,
	PREU_BRUT = pPREU_BRUT,
	BENEFICI_NET = pBENEFICI_NET
    WHERE ID_BOTIGA = pID_BOTIGA
          AND ID_PROD = pID_PROD
          AND ID_DATA = pID_DATA
          AND ID_HORA = pID_HORA;
		
    ALTALOG ('MODIFICARDWHVENDA',PARAMS,RESPOSTA);

  EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap dwh venda amb identificador proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   ALTALOG ('MODIFICARDWHVENDA',PARAMS,RESPOSTA);
		   
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci de la venda : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
		  ALTALOG ('MODIFICARDWHVENDA',PARAMS,RESPOSTA);
  END;

COMMIT;
END;
/


/*****************************************************************
-- Nom:                 ESBORRARDWHVENDA
--
-- Descripci:          Esborra les dades d'una dwh venda
--
-- Tasca: CU15-03
--
-- Parmetres entrada:
						pID_BOTIGA VENDA.ID_BOTIGA%TYPE,
						pID_PROD VENDA.ID_PROD%TYPE,
						pID_DATA DWH_VENDA.ID_DATA%TYPE,
						pID_HORA DWH_VENDA.ID_HORA%TYPE,
						pQUANTITAT DWH_VENDA.QUANTITAT%TYPE,
						pPREU_BRUT DWH_VENDA.PREU_BRUT%TYPE,
						pBENEFICI_NET DWH_VENDA.BENEFICI_NET%TYPE,

--
-- Error controlats:   dwh venda no trobada
--
*****************************************************************/
CREATE OR REPLACE PROCEDURE "ESBORRARDWHVENDA" ( 
						pID_BOTIGA DWH_VENDA.ID_BOTIGA%TYPE,
						pID_PROD DWH_VENDA.ID_PROD%TYPE,
						pID_DATA DWH_VENDA.ID_DATA%TYPE,
						pID_HORA DWH_VENDA.ID_HORA%TYPE,
					        RESPOSTA OUT NOCOPY VARCHAR2)  
  AS
  COUNT_TMP INTEGER;
  PARAMS VARCHAR2(1024);

BEGIN
  
  BEGIN
    RESPOSTA:='OK';

    PARAMS:=pID_BOTIGA||';'||pID_PROD||';'||pID_DATA||';'||pID_HORA;

    -- El primer que fem es comprovar si existeix ja un registre amb la mateixa PK.

    SELECT COUNT(*) INTO COUNT_TMP FROM DWH_VENDA WHERE ID_BOTIGA = pID_BOTIGA
                                             		AND ID_PROD = pID_PROD
                                               		AND ID_DATA = pID_DATA
                                               		AND ID_HORA = pID_HORA;

     IF (COUNT_TMP = 0) THEN
       	RAISE NO_DATA_FOUND;
     END IF;
    
     DELETE FROM DWH_VENDA WHERE ID_BOTIGA = pID_BOTIGA
                       		AND ID_PROD = pID_PROD
                       		AND ID_DATA = pID_DATA
                       		AND ID_HORA = pID_HORA;

     ALTALOG ('ESBORRARDWHVENDA',PARAMS,RESPOSTA);

  EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap dwh venda amb identificador proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   ALTALOG ('ESBORRARDWHVENDA',PARAMS,RESPOSTA);
        
        WHEN OTHERS THEN
          RESPOSTA := 'Error esborrant dwh venda : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA); 
		  ALTALOG ('ESBORRARDWHVENDA',PARAMS,RESPOSTA);		  
  END;

COMMIT;
END;
/
