---------------------------------------------------------------------------------
-- Alumne: Miguel Alvarez Lopez (bluesea)
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO D'UN DATA WAREHOUSE PER UNA CADENA DE BOTIGUES DE ROBA
---------------------------------------------------------------------------------


/*****************************************************************
-- Nom:                 MOD_EST_I
--
-- Descripci:          modificar DWH_EST amb les consultes del 
--                      mdul estadstiques, per un any
--
-- Parmetres entrada:
						pANYO DWH_EST.ANYO%TYPE
--
-- Error controlats:  - ANY duplicat 
--
*****************************************************************/
CREATE OR REPLACE PROCEDURE "MOD_EST_I" ( 
                                        pANYO DWH_EST.ANYO%TYPE,
                                        RESPOSTA OUT NOCOPY VARCHAR2)

  AS
  PARAMS VARCHAR2(1024);

BEGIN
  RESPOSTA:='OK';
  DECLARE
  	pBNET_CAD		NUMBER;
	pBOTIGA_MES_BNET	NUMBER(3);
	pBNET_BOT		NUMBER;
	pID_PROD		VARCHAR2(13);
	pQUANT_PROD		NUMBER;
	pHORA_MES_PROD		VARCHAR2(50);
	pQ_HORA_MES_PROD	NUMBER;
	pHORA_MENYS_PROD	VARCHAR2(50);
	pQ_HORA_MENYS_PROD	NUMBER;
	pDIA_MES_VENDA		VARCHAR2(50);
	pQ_DIA_MES_VENDA	NUMBER;
	pDIA_MENYS_VENDA	VARCHAR2(50);
	pQ_DIA_MENYS_VENDA	NUMBER;
	pCIUTAT_MES_BNET	VARCHAR2(25);
	pBNET_CIUTAT		NUMBER;
	pTPC_BNET_BVIRTUALS	NUMBER;
	ANNY			NUMBER(4);

        pHora DWH_VENDA.ID_HORA%TYPE;
        pQHora DWH_VENDA.QUANTITAT%TYPE; 
        pDia VARCHAR2(2);
        pQDia DWH_VENDA.QUANTITAT%TYPE; 

  CURSOR cHora_Mes_Prod IS
    SELECT
        DV2.ID_HORA,
        SUM(DV2.QUANTITAT)
    FROM DWH_VENDA DV2, 
       (SELECT 
            ANYO,
            MAX(QUANTITATHORA) AS MAX_HORA
        FROM
           (SELECT
               TO_CHAR(DV3.ID_DATA,'YYYY') AS ANYO,
               DV3.ID_HORA AS HORA,
               SUM(DV3.QUANTITAT) AS QUANTITATHORA
            FROM DWH_VENDA DV3
            WHERE TO_CHAR(DV3.ID_DATA,'YYYY') = pANYO
            GROUP BY 
               TO_CHAR(DV3.ID_DATA,'YYYY'),
               DV3.ID_HORA
            )
        GROUP BY 
            ANYO
         ) T3
    WHERE TO_CHAR(DV2.ID_DATA,'YYYY') = T3.ANYO
    GROUP BY 
          DV2.ID_HORA,
          T3.MAX_HORA
    HAVING SUM(DV2.QUANTITAT) = T3.MAX_HORA;

  CURSOR cHora_Menys_Prod IS
    SELECT
        DV2.ID_HORA,
        SUM(DV2.QUANTITAT)
    FROM DWH_VENDA DV2, 
       (SELECT 
            ANYO,
            MIN(QUANTITATHORA) AS MIN_HORA
        FROM
           (SELECT
               TO_CHAR(DV3.ID_DATA,'YYYY') AS ANYO,
               DV3.ID_HORA AS HORA,
               SUM(DV3.QUANTITAT) AS QUANTITATHORA
            FROM DWH_VENDA DV3
            WHERE TO_CHAR(DV3.ID_DATA,'YYYY') = pANYO
            GROUP BY 
               TO_CHAR(DV3.ID_DATA,'YYYY'),
               DV3.ID_HORA
            )
        GROUP BY 
            ANYO
         ) T3
    WHERE TO_CHAR(DV2.ID_DATA,'YYYY') = T3.ANYO
    GROUP BY 
          DV2.ID_HORA,
          T3.MIN_HORA
    HAVING SUM(DV2.QUANTITAT) = T3.MIN_HORA;

  CURSOR cDia_Mes_Venda IS
    SELECT
       TO_CHAR(DV2.ID_DATA,'DD'),
       SUM(DV2.QUANTITAT)
    FROM DWH_VENDA DV2, 
       (SELECT 
           ANYO,
           MAX(QUANTITATDIA) AS MAX_DIA
       FROM
           (SELECT
               TO_CHAR(DV3.ID_DATA,'YYYY') AS ANYO,
               TO_CHAR(DV3.ID_DATA,'DD') AS DIA,
               SUM(DV3.QUANTITAT) AS QUANTITATDIA
           FROM DWH_VENDA DV3
           WHERE TO_CHAR(DV3.ID_DATA,'YYYY') = pANYO
           GROUP BY 
              TO_CHAR(DV3.ID_DATA,'YYYY'),
              TO_CHAR(DV3.ID_DATA,'DD')
           )
       GROUP BY 
          ANYO) T3
    WHERE TO_CHAR(DV2.ID_DATA,'YYYY') = T3.ANYO
    GROUP BY 
       TO_CHAR(DV2.ID_DATA,'DD'),
       T3.MAX_DIA
    HAVING SUM(DV2.QUANTITAT) = T3.MAX_DIA;

  CURSOR cDia_Menys_Venda IS
    SELECT
       TO_CHAR(DV2.ID_DATA,'DD'),
       SUM(DV2.QUANTITAT)
    FROM DWH_VENDA DV2, 
       (SELECT 
           ANYO,
           MIN(QUANTITATDIA) AS MIN_DIA
       FROM
           (SELECT
               TO_CHAR(DV3.ID_DATA,'YYYY') AS ANYO,
               TO_CHAR(DV3.ID_DATA,'DD') AS DIA,
               SUM(DV3.QUANTITAT) AS QUANTITATDIA
           FROM DWH_VENDA DV3
           WHERE TO_CHAR(DV3.ID_DATA,'YYYY') = pANYO
           GROUP BY 
              TO_CHAR(DV3.ID_DATA,'YYYY'),
              TO_CHAR(DV3.ID_DATA,'DD')
           )
       GROUP BY 
          ANYO) T3
    WHERE TO_CHAR(DV2.ID_DATA,'YYYY') = T3.ANYO
    GROUP BY 
       TO_CHAR(DV2.ID_DATA,'DD'),
       T3.MIN_DIA
    HAVING SUM(DV2.QUANTITAT) = T3.MIN_DIA;


  BEGIN

  -- Fem les consultes del mdul estadstic i les asignem a les variables corresponents.
  -- ===================================================================================

  -- Consulta per obtenir e benefici net total de la cadena. 
  -- Tasca: CU17-01
  SELECT 
    SUM(DV1.QUANTITAT * DV1.BENEFICI_NET)
  INTO pBNET_CAD
  FROM DWH_VENDA DV1
  WHERE  TO_CHAR(DV1.ID_DATA,'YYYY') = pANYO;
    DBMS_OUTPUT.PUT_LINE('SQL1 = ' || pBNET_CAD);

 -- Consulta per obtenir el ID de la botiga amb mes benefici net i la quantitat.
 -- Tasca: CU17-03
  SELECT
    DV2.ID_BOTIGA,
    SUM(DV2.QUANTITAT * DV2.BENEFICI_NET)
  INTO
    pBOTIGA_MES_BNET,
    pBNET_BOT
  FROM DWH_VENDA DV2,
    (SELECT 
        MAX(BNETBOTIGA) AS MAX_BOTIGA
     FROM
       (SELECT
          DV.ID_BOTIGA AS BOTIGA,
          SUM(DV.QUANTITAT * DV.BENEFICI_NET) AS BNETBOTIGA
        FROM DWH_VENDA DV
        WHERE TO_CHAR(DV.ID_DATA,'YYYY') = pANYO
        GROUP BY 
          DV.ID_BOTIGA
        )
     ) T3
  GROUP BY 
    TO_CHAR(DV2.ID_DATA,'YYYY'),
    DV2.ID_BOTIGA,
    T3.MAX_BOTIGA
  HAVING SUM(DV2.QUANTITAT * DV2.BENEFICI_NET) = T3.MAX_BOTIGA;
DBMS_OUTPUT.PUT_LINE('SQL2 = ' || pBOTIGA_MES_BNET ||' SQL22: '|| pBNET_BOT);

 -- Consulta per obtenir l'identificador EAN13 del prod. i quantitat venuda.
 -- Tasca: CU18-01
  SELECT
    DV2.ID_PROD,
    SUM(DV2.QUANTITAT)
  INTO
    pID_PROD,
    pQUANT_PROD
  FROM DWH_VENDA DV2,
      (SELECT 
         MAX(QUANTITAT) AS MAX_QUANTITAT
       FROM
        (SELECT
           DV.ID_PROD AS BOTIGA,
           SUM(DV.QUANTITAT) AS QUANTITAT
         FROM DWH_VENDA DV
         WHERE TO_CHAR(DV.ID_DATA,'YYYY') = pANYO  
         GROUP BY 
            DV.ID_PROD
         )
      ) t3
  GROUP BY 
    TO_CHAR(DV2.ID_DATA,'YYYY'),
    DV2.ID_PROD,
    T3.MAX_QUANTITAT
  HAVING SUM(DV2.QUANTITAT) =  T3.MAX_QUANTITAT;
    DBMS_OUTPUT.PUT_LINE('SQL3 = ' || pID_PROD||' SQL33: '|| pQUANT_PROD);

 -- Consulta per obtenir l'hora del dia on ms producte s'ha venut i la quantitat.
 -- Tasca: CU18-02
 -- IMPORTANT: aquesta consulta la fem implementant cursors ja que potser que el resultat
 -- d'aquesta dongui mes d'un valor (empat). 
 -- Exemple: poden haver varies hores en que es vengui la mateixa quanitat mxima de 
 -- producte.

  pHORA_MES_PROD := '';

    OPEN cHora_Mes_Prod;
    LOOP
        FETCH cHora_Mes_Prod into pHora, pQHora;
        EXIT WHEN cHora_Mes_Prod%NOTFOUND;
        pHORA_MES_PROD  := pHORA_MES_PROD||TO_CHAR(pHora)||';';
        
    END LOOP;
    pHORA_MES_PROD := SUBSTR(pHORA_MES_PROD,1,LENGTH(pHORA_MES_PROD)-1);
    pQ_HORA_MES_PROD := pQHora;
    CLOSE cHora_Mes_Prod;

    DBMS_OUTPUT.PUT_LINE('SQL4 = ' || pHORA_MES_PROD||' SQL44: '|| pQ_HORA_MES_PROD);

 -- Consulta per obtenir l'hora del dia on menys producte s'ha venut i la quantitat.
 -- Tasca: CU18-03
 -- IMPORTANT: aquesta consulta la fem implementant cursors ja que potser que el resultat
 -- d'aquesta dongui mes d'un valor (empat). 
 -- Exemple: poden haver varies hores en que es vengui la mateixa quanitat mnima de 
 -- producte.

  pHORA_MENYS_PROD := '';

    OPEN cHora_Menys_Prod;
    LOOP
        FETCH cHora_Menys_Prod into pHora, pQHora;
        EXIT WHEN cHora_Menys_Prod%NOTFOUND;
        pHORA_MENYS_PROD  := pHORA_MENYS_PROD||TO_CHAR(pHora)||';';
        
    END LOOP;
    pHORA_MENYS_PROD := SUBSTR(pHORA_MENYS_PROD,1,LENGTH(pHORA_MENYS_PROD)-1);
    pQ_HORA_MENYS_PROD := pQHora;
    CLOSE cHora_Menys_Prod;

    DBMS_OUTPUT.PUT_LINE('SQL5 = ' || pHORA_MENYS_PROD||' SQL55: '|| pQ_HORA_MENYS_PROD);

 -- Consulta per obtenir el dia del mes on ms vendes s'han realitzat i la quantitat.
 -- Tasca: CU19-01
 -- IMPORTANT: aquesta consulta la fem implementant cursors ja que potser que el resultat
 -- d'aquesta dongui mes d'un valor (empat). 
 -- Exemple: poden haver diversos dies en que es vengui la mateixa quanitat mnima de 
 -- producte.
 
  pDIA_MES_VENDA := '';

    OPEN cDia_Mes_Venda;
    LOOP
        FETCH cDia_Mes_Venda into pDia, pQDia;
        EXIT WHEN cDia_Mes_Venda%NOTFOUND;
        pDIA_MES_VENDA  := pDIA_MES_VENDA||pDia||';';
        
    END LOOP;
    pDIA_MES_VENDA := SUBSTR(pDIA_MES_VENDA,1,LENGTH(pDIA_MES_VENDA)-1);
    pQ_DIA_MES_VENDA := pQDia;
    CLOSE cDia_Mes_Venda;

    DBMS_OUTPUT.PUT_LINE('SQL6 = ' || pDIA_MES_VENDA||' SQL66: '|| pQ_DIA_MES_VENDA);

 -- Consulta per obtenir el dia del mes on menys vendes s'han realitzat i la quantitat.
 -- Tasca: CU19-02
 -- IMPORTANT: aquesta consulta la fem implementant cursors ja que potser que el resultat
 -- d'aquesta dongui mes d'un valor (empat). 
 -- Exemple: poden haver varies hores en que es vengui la mateixa quanitat mnima de 
 -- producte.

  pDIA_MENYS_VENDA := '';

    OPEN cDia_Menys_Venda;
    LOOP
        FETCH cDia_Menys_Venda into pDia, pQDia;
        EXIT WHEN cDia_Menys_Venda%NOTFOUND;
        pDIA_MENYS_VENDA  := pDIA_MENYS_VENDA||pDia||';';
        
    END LOOP;
    pDIA_MENYS_VENDA := SUBSTR(pDIA_MENYS_VENDA,1,LENGTH(pDIA_MENYS_VENDA)-1);
    pQ_DIA_MENYS_VENDA := pQDia;
    CLOSE cDia_Menys_Venda;

    DBMS_OUTPUT.PUT_LINE('SQL7 = ' || pDIA_MENYS_VENDA||' SQL77: '|| pQ_DIA_MENYS_VENDA);

 -- Consulta per obtenir la ciutat amb mes benefici net obtinguts i la quantitat.
 -- Tasca: CU17-02
  SELECT
    C2.DESCRIPCIO,
    SUM(DV2.QUANTITAT * DV2.BENEFICI_NET)
  INTO
    pCIUTAT_MES_BNET,
    pBNET_CIUTAT
  FROM DWH_VENDA DV2, BOTIGA B2, CIUTAT C2,
    (SELECT 
        MAX(BNETCIUTAT) AS MAX_CIUTAT
    FROM
       (SELECT
          B.ID_CIUTAT,
          SUM(DV.QUANTITAT * DV.BENEFICI_NET) AS BNETCIUTAT
        FROM DWH_VENDA DV, BOTIGA B
        WHERE TO_CHAR(DV.ID_DATA,'YYYY') = pANYO
          AND DV.ID_BOTIGA = B.ID_BOTIGA
        GROUP BY 
          B.ID_REGIO,
          B.ID_CIUTAT
        )
     ) T3
  WHERE DV2.ID_BOTIGA = B2.ID_BOTIGA
        AND B2.ID_REGIO = C2.ID_REGIO
        AND B2.ID_CIUTAT = C2.ID_CIUTAT
  GROUP BY 
    TO_CHAR(DV2.ID_DATA,'YYYY'),
    C2.DESCRIPCIO,
    B2.ID_REGIO,
    B2.ID_CIUTAT,
    T3.MAX_CIUTAT
  HAVING SUM(DV2.QUANTITAT * DV2.BENEFICI_NET) = T3.MAX_CIUTAT;
  DBMS_OUTPUT.PUT_LINE('SQL8 = ' || pCIUTAT_MES_BNET||' SQL88: '|| pBNET_CIUTAT);

 -- Consulta per obtenir el % de benefici net de les tendes virtuals envers
 -- el total del benefici net de la cadena.
 -- Tasca: CU17-04
  SELECT 
     ROUND((SUM(dv1.benefici_net * dv1.quantitat) * 100) / (select SUM(dv2.benefici_net * dv2.quantitat)
                                                     FROM dwh_venda dv2
                                                     WHERE  TO_CHAR(dv2.id_data,'YYYY') = pANYO),2)
  INTO
    pTPC_BNET_BVIRTUALS
  FROM dwh_venda dv1, botiga b1
  WHERE TO_CHAR(dv1.id_data,'YYYY') = pANYO
      AND b1.id_botiga = dv1.id_botiga
      AND b1.fisica = 'N'
  GROUP BY TO_CHAR(dv1.id_data,'YYYY');
  DBMS_OUTPUT.PUT_LINE(' SQL99: '|| pTPC_BNET_BVIRTUALS);

  -- Modifiquem la taula DWH_EST amb les dades estadstiques obtingudes. 
  -- Aix ho fem per a un any concret, el de la venda realitzada (parmetre).
  
  UPDATE DWH_EST
  SET BNET_CAD = pBNET_CAD,
	BOTIGA_MES_BNET = pBOTIGA_MES_BNET,
	BNET_BOT = pBNET_BOT,
	ID_PROD = pID_PROD,
	QUANT_PROD = pQUANT_PROD,
	HORA_MES_PROD = pHORA_MES_PROD,
	Q_HORA_MES_PROD = pQ_HORA_MES_PROD,
	HORA_MENYS_PROD = pHORA_MENYS_PROD,
	Q_HORA_MENYS_PROD = pQ_HORA_MENYS_PROD,
	DIA_MES_VENDA = pDIA_MES_VENDA,
	Q_DIA_MES_VENDA = pQ_DIA_MES_VENDA,
	DIA_MENYS_VENDA = pDIA_MENYS_VENDA,
	Q_DIA_MENYS_VENDA = pQ_DIA_MENYS_VENDA,
	CIUTAT_MES_BNET = pCIUTAT_MES_BNET,
	BNET_CIUTAT = pBNET_CIUTAT,
	TPC_BNET_BVIRTUALS = pTPC_BNET_BVIRTUALS
  WHERE ANYO = pANYO;

  ALTALOG ('MOD_EST_I',PARAMS,RESPOSTA);

  EXCEPTION  
        WHEN NO_DATA_FOUND THEN
           RESPOSTA := 'No existeix cap any amb identificador proporcionat.';
           DBMS_OUTPUT.PUT_LINE(RESPOSTA);
		   ALTALOG ('MOD_EST_I',PARAMS,RESPOSTA);
		   
        WHEN OTHERS THEN
          RESPOSTA := 'Error en la modificaci de dwh est : (' || SQLCODE || ') ' || SQLERRM;
          DBMS_OUTPUT.PUT_LINE(RESPOSTA);     
		  ALTALOG ('MOD_EST_I',PARAMS,RESPOSTA);
  END;

COMMIT;
END;
/

